import 'package:bento_support_flutter/UI/view/chat/chat_screen.dart';
import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/models/socket_message.dart';
import 'package:bento_support_flutter/services/socket.service.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:rothko_mobile/rothko.dart';
import 'package:timeago/timeago.dart' as timeago;

enum UserType { anonymous, client, agent }

class GeneralBubble extends StatelessWidget {
  final Message<Profile> chatMessage;

  GeneralBubble(this.chatMessage);

  final chatController = Get.find<ChatController>();

  @override
  Widget build(BuildContext context) {
    print(chatMessage.review);
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        if (chatMessage.type == MessageType.complaintResolved)
          Padding(
            padding: EdgeInsets.fromLTRB(24, 0, 24, 10),
            child: Container(
              decoration: BoxDecoration(
                color: kLightNeutralBase,
                borderRadius: BorderRadius.circular(8),
              ),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  SizedBox(height: 8),
                  Text(
                    'Were you satisfied with our support on this issue?',
                    style: TextStyle(
                      fontFamily: 'Jost',
                      fontSize: 12,
                      fontWeight: FontWeight.w400,
                      color: kDarkNeutralLighten1,
                    ),
                  ),
                  Container(
                    margin: EdgeInsets.symmetric(vertical: 10.0),
                    padding: EdgeInsets.symmetric(horizontal: 24),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        GestureDetector(
                          child: FeelingsWidget(
                              imagePath:
                                  'https://verifi-app-bucket.s3.amazonaws.com/terrible.png',
                              text: 'Terrible',
                              selected: chatMessage.review == 1,
                              color: kErrorBase),
                          onTap: () => chatController.reviewMessage(
                              chatMessage, 1, context),
                        ),
                        GestureDetector(
                          child: FeelingsWidget(
                              imagePath:
                                  'https://verifi-app-bucket.s3.amazonaws.com/okay.png',
                              text: 'Okay',
                              selected: chatMessage.review == 2,
                              color: kWarningBase),
                          onTap: () => chatController.reviewMessage(
                              chatMessage, 2, context),
                        ),
                        GestureDetector(
                          child: FeelingsWidget(
                              imagePath:
                                  'https://verifi-app-bucket.s3.amazonaws.com/good.png',
                              selected: chatMessage.review == 3,
                              text: 'Good',
                              color: kSuccessDarken4),
                          onTap: () => chatController.reviewMessage(
                              chatMessage, 3, context),
                        ),
                        GestureDetector(
                          child: FeelingsWidget(
                              imagePath:
                                  'https://verifi-app-bucket.s3.amazonaws.com/fantastic.png',
                              text: 'Fantastic',
                              selected: chatMessage.review == 4,
                              color: kPrimaryBase),
                          onTap: () => chatController.reviewMessage(
                              chatMessage, 4, context),
                        )
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        if (chatMessage.type == MessageType.newComplaint)
          Row(
            children: [
              Expanded(
                  child: Divider(
                endIndent: 18,
              )),
              Text('new complaint',
                  style: TextStyle(
                      color: kDarkNeutralLighten3, fontFamily: 'jost')),
              Expanded(
                  child: Divider(
                indent: 18,
              ))
            ],
          ),
        if (chatMessage.body != null && chatMessage.body!.isNotEmpty)
          ChatBubble(chatMessage),
        if (chatMessage.attachments.isNotEmpty) ...[
          SizedBox(height: 10),
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: chatMessage.sender == MessageSender.client
                ? MainAxisAlignment.end
                : MainAxisAlignment.start,
            children: [
              if (chatMessage.sender == MessageSender.user) ...[
                chatMessage.senderAvatar,
                SizedBox(width: 18)
              ],
              Column(
                  crossAxisAlignment: chatMessage.sender == MessageSender.client
                      ? CrossAxisAlignment.end
                      : CrossAxisAlignment.start,
                  children: [
                    ...chatMessage.attachments
                        .map((e) =>
                            e.getPreviewWidget(chatMessage.sender, context))
                        .toList(),
                    SizedBox(
                      height: 8,
                    ),
                    chatMessage.timeAgoWidget,
                  ]),
              if (chatMessage.sender == MessageSender.client) ...[
                SizedBox(width: 18),
                chatMessage.senderAvatar
              ],
            ],
          ),
          SizedBox(height: 8),
        ]
      ],
    );
  }
}

class ChatBubble extends StatelessWidget {
  final Message<Profile> chatMessage;

  const ChatBubble(this.chatMessage);
  @override
  Widget build(BuildContext context) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: chatMessage.sender == MessageSender.user
          ? MainAxisAlignment.start
          : MainAxisAlignment.end,
      children: [
        if (chatMessage.sender == MessageSender.user) ...[
          chatMessage.senderAvatar,
          SizedBox(width: 18),
        ],
        Column(
          crossAxisAlignment: chatMessage.sender == MessageSender.user
              ? CrossAxisAlignment.start
              : CrossAxisAlignment.end,
          children: [
            Container(
              padding: EdgeInsets.all(12),
              width: MediaQuery.of(context).size.width * 0.6,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  color: chatMessage.sender == MessageSender.user
                      ? kLightNeutralBase
                      : kPrimaryBase),
              child: Text(
                chatMessage.body ?? '',
                style: TextStyle(
                    fontFamily: 'Jost',
                    color: chatMessage.sender == MessageSender.user
                        ? Colors.black
                        : Colors.white),
              ),
            ),
            SizedBox(height: 8),
            if (chatMessage.attachments.isEmpty) chatMessage.timeAgoWidget,
          ],
        ),
        if (chatMessage.sender == MessageSender.client) ...[
          SizedBox(width: 18),
          chatMessage.senderAvatar,
        ],
      ],
    );
  }
}
