import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/controllers/auth_controller.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:rothko_mobile/rothko_system/utilities/colors.dart';

class SupportBackground extends StatefulWidget {
  final bool normalAppbar;
  final Widget? body;

  SupportBackground({
    Key? key,
    this.normalAppbar = false,
    this.body,
  }) : super(key: key);

  @override
  _SupportBackgroundState createState() => _SupportBackgroundState();
}

class _SupportBackgroundState extends State<SupportBackground> {
  var authController = Get.find<AuthController>();
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Color(0xffeff2f8),
      body: Stack(
        children: [
          if (!widget.normalAppbar)
            BuildUpper(
              normalAppbar: widget.normalAppbar,
            ),
          Column(children: [
            BuildMenu(
              backgroundColor:
                  widget.normalAppbar ? kPrimaryBase : Colors.transparent,
            ),
            SizedBox(height: 20),
            Expanded(
              child: Builder(builder: (context) {
                if (widget.body != null) {
                  return widget.body!;
                }
                return Container();
              }),
            )
          ]),
        ],
      ),
    );
  }
}

class BuildMenu extends StatelessWidget {
  BuildMenu({Key? key, required this.backgroundColor}) : super(key: key);

  final authController = Get.find<AuthController>();
  final chatController = Get.find<ChatController>();
  final Color backgroundColor;
  @override
  Widget build(BuildContext context) {
    return Container(
      width: MediaQuery.of(context).size.width,
      padding: EdgeInsets.only(
          left: 10,
          right: 26,
          top: MediaQuery.of(context).padding.top + 8,
          bottom: 30),
      color: backgroundColor,
      child: Builder(builder: (context) {
        return Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            InkWell(
                child: Icon(
                  Icons.arrow_back,
                  size: 24,
                  color: Colors.white,
                ),
                onTap: () {
                  Get.find<ChatController>().expanded.value = false;
                }),
            SizedBox(width: 16),
            Stack(
              children: [
                Container(
                  height: 40,
                  width: 40,
                  decoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: kPrimaryLighten0,
                  ),
                  padding: EdgeInsets.all(2),
                  child: Image.network(
                      authController.orgDefaultResponder.value.avatarUrl),
                ),
                Positioned(
                    bottom: 0,
                    right: 0,
                    child: Container(
                      height: 10,
                      width: 10,
                      decoration: BoxDecoration(
                          shape: BoxShape.circle, color: kSuccessBase),
                    ))
              ],
            ),
            SizedBox(width: 16),
            Obx(() => Column(
                  children: [
                    Text(
                      chatController.currentAgent.value.name,
                      style: TextStyle(
                          fontWeight: FontWeight.w500,
                          fontSize: 16.0,
                          color: kPrimaryLighten0,
                          fontFamily: 'Jost'),
                    ),
                    Text(
                      'Active',
                      style: TextStyle(
                          fontWeight: FontWeight.w400,
                          fontSize: 12.0,
                          color: kPrimaryLighten0,
                          fontFamily: 'Jost'),
                    ),
                  ],
                ))
          ],
        );
      }),
    );
  }
}

class BuildUpper extends StatelessWidget {
  final bool normalAppbar;
  const BuildUpper({
    Key? key,
    this.normalAppbar = false,
  }) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Container(
          height: MediaQuery.of(context).size.height *
              ((normalAppbar ? 15 : 40) / 100),
          decoration: BoxDecoration(
              borderRadius: BorderRadius.only(
                bottomRight:
                    normalAppbar ? Radius.circular(0) : Radius.circular(50),
                bottomLeft:
                    normalAppbar ? Radius.circular(0) : Radius.circular(50),
              ),
              color: Color(0xff0047CC)),
          child: Stack(
            children: <Widget>[
              Positioned.fill(
                top: -((MediaQuery.of(context).size.height * (40 / 100)) / 2),
                bottom: returnPercentageValue(
                        MediaQuery.of(context).size.height, (100 - 40))! /
                    4,
                left: MediaQuery.of(context).size.width / 3,
                right: -(MediaQuery.of(context).size.width / 2),
                child: Container(
                    height: returnPercentageValue(
                        MediaQuery.of(context).size.height, (100 - 40)),
                    decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        color: Colors.white.withOpacity(0.08))),
              ),
            ],
          ),
        ),
      ],
    );
  }

  double? returnPercentageValue(num x, num xpercentage) {
    return x * (xpercentage / 100);
  }
}
