import 'dart:io';

import 'package:bento_support_flutter/UI/shared/custom_widgets/chat_bubble.dart';
import 'package:bento_support_flutter/UI/shared/custom_widgets/loading_widget.dart';
import 'package:bento_support_flutter/UI/shared/custom_widgets/support_background.dart';
import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/UI/view/support.dart';
import 'package:bento_support_flutter/controllers/auth_controller.dart';
import 'package:bento_support_flutter/services/parser.service.dart';
import 'package:emoji_picker_flutter/emoji_picker_flutter.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:rothko_mobile/rothko.dart';

class ChatScreen extends StatelessWidget {
  final chatController = Get.find<ChatController>();
  final authController = Get.find<AuthController>();

  final loading = Scaffold(
    backgroundColor: Colors.white,
    body: LoadingWidget(),
  );

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      if (!authController.isInitialized.value) {
        return loading;
      }
      return SupportBackground(
        normalAppbar:
            chatController.currentState.value == ChatState.selectTagView
                ? false
                : true,
        body: Obx(() {
          switch (chatController.currentState.value) {
            case ChatState.anonymousMainView:
              {
                return AnonymousFirstView();
              }
            case ChatState.startConversationView:
              {
                return StartConversationView();
              }
            case ChatState.selectTagView:
              {
                return SupportScreen();
              }
            default:
              {
                return loading;
              }
          }
        }),
      );
    });
  }
}

class AnonymousFirstView extends StatelessWidget {
  final chatController = Get.find<ChatController>();
  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
        reverse: true,
        child: Column(
          children: [
            Obx(() => Column(
                  children: List.generate(chatController.messages.length,
                          (int index) {
                    return index;
                  })
                      .map((x) => Padding(
                          padding: EdgeInsets.symmetric(
                              vertical: 18, horizontal: 22),
                          child: Obx(() =>
                              GeneralBubble(chatController.messages[x].value))))
                      .toList(),
                )),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 24.0),
              child: Container(
                decoration: BoxDecoration(
                  color: kLightNeutralBase,
                  borderRadius: BorderRadius.circular(8),
                ),
                padding: EdgeInsets.symmetric(horizontal: 16.0, vertical: 10),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Text(
                          'Are you an Existing User?',
                          style: TextStyle(
                            fontFamily: 'Jost',
                            fontSize: 14,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                        SizedBox(height: 6),
                        Button(
                          filled: true,
                          text: 'Login',
                          icon: '',
                          fontSize: 11,
                          onPressed: chatController.login,
                        )
                      ],
                    ),
                    Text(
                      'Personalize your complaints to your organization',
                      style: TextStyle(
                        fontFamily: 'Jost',
                        fontSize: 12,
                        fontWeight: FontWeight.w400,
                        color: kDarkNeutralLighten1,
                      ),
                    )
                  ],
                ),
              ),
            ),
            SizedBox(
              height: 18,
            ),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 24.0),
              child: Container(
                decoration: BoxDecoration(
                  color: kLightNeutralBase,
                  borderRadius: BorderRadius.circular(8),
                ),
                padding: EdgeInsets.symmetric(horizontal: 16.0, vertical: 10),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Text(
                          'New User',
                          style: TextStyle(
                            fontFamily: 'Jost',
                            fontSize: 14,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                        SizedBox(height: 6),
                        Button(
                          filled: false,
                          text: 'Start Conversation',
                          icon: '',
                          fontSize: 11,
                          onPressed: () {
                            chatController.startConversationView();
                          },
                        )
                      ],
                    ),
                    Text(
                      'I just want to have a conversation',
                      style: TextStyle(
                        fontFamily: 'Jost',
                        fontSize: 12,
                        fontWeight: FontWeight.w400,
                        color: kDarkNeutralLighten1,
                      ),
                    )
                  ],
                ),
              ),
            ),
            SizedBox(
              height: 18,
            ),
          ],
        ));
  }
}

class StartConversationView extends StatefulWidget {
  @override
  State<StartConversationView> createState() => _StartConversationViewState();
}

class _StartConversationViewState extends State<StartConversationView> {
  final chatController = Get.find<ChatController>();

  final TextEditingController controller = TextEditingController();

  final dateList = <String>[];

  void _onEmojiSelected(Emoji emoji) {
    controller
      ..text += emoji.emoji
      ..selection = TextSelection.fromPosition(
          TextPosition(offset: controller.text.length));
  }

  void _onBackspacePressed() {
    controller
      ..text = controller.text.characters.skipLast(1).toString()
      ..selection = TextSelection.fromPosition(
          TextPosition(offset: controller.text.length));
  }

  @override
  Widget build(BuildContext context) {
    dateList.clear();
    return Column(
      children: [
        Expanded(
          child: Obx(
            () {
              return SmartRefresher(
                onRefresh: () =>
                    chatController.fetchMessages(loadPreviousMessages: true),
                onLoading: () =>
                    chatController.fetchMessages(loadPreviousMessages: false),
                enablePullUp: true,
                enablePullDown: true,
                footer: SliverList(
                  delegate: SliverChildBuilderDelegate(
                      (BuildContext context, int index) {
                    return chatController.conversationViewController.isLoading
                        ? Center(
                            child: SizedBox(
                                height: 14,
                                width: 14,
                                child: CircularProgressIndicator(
                                  strokeWidth: 2,
                                )))
                        : SizedBox(
                            height: 2,
                          );
                  }, childCount: 1),
                ),
                controller: chatController.conversationViewController,
                child: ListView.builder(
                    padding: EdgeInsets.only(
                        top: 135, bottom: 100, left: 18, right: 18),
                    reverse: false,
                    controller: chatController.chatScreenScrollController,
                    shrinkWrap: true,
                    itemCount: chatController.messages.length,
                    itemBuilder: (context, index) {
                      return Padding(
                          padding: EdgeInsets.symmetric(vertical: 18),
                          child: Builder(builder: (c) {
                            var dateSent = ParserService.parseDateToString(
                                chatController.messages[index].value.createdAt,
                                'dd MMMM yyyy');
                            if (!dateList.contains(dateSent)) {
                              dateList.add(dateSent);
                              return Column(
                                children: [
                                  Text(dateSent,
                                      style: TextStyle(
                                          color: kDarkNeutralLighten3,
                                          fontFamily: 'jost')),
                                  GeneralBubble(
                                      chatController.messages[index].value)
                                ],
                              );
                            }
                            return GeneralBubble(
                                chatController.messages[index].value);
                          }));
                    }),
              );
            },
          ),
        ),
        Container(
            child: Padding(
          padding: EdgeInsets.symmetric(horizontal: 24.0, vertical: 10),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              Container(
                decoration: BoxDecoration(
                  color: kLightNeutralBase,
                  borderRadius: BorderRadius.circular(100),
                ),
                padding: EdgeInsets.symmetric(
                  horizontal: 10.0,
                ),
                constraints: BoxConstraints(maxHeight: 150),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Row(
                      children: [
                        Expanded(
                          child: TextFormField(
                            maxLines: 4,
                            minLines: 1,
                            controller: controller,
                            onTap: () => chatController.showEmoji.value = false,
                            decoration: InputDecoration(
                              hintText: 'Talk to us...',
                              focusedBorder: OutlineInputBorder(
                                  borderSide:
                                      BorderSide(color: Colors.transparent),
                                  borderRadius: BorderRadius.circular(120)),
                              enabledBorder: OutlineInputBorder(
                                borderRadius: BorderRadius.circular(100),
                                borderSide:
                                    BorderSide(color: Colors.transparent),
                              ),
                              hintStyle: TextStyle(color: Colors.black54),
                            ),
                          ),
                        ),
                        SizedBox(
                          width: 15,
                        ),

                        InkWell(
                          onTap: () {
                            FocusScope.of(context).unfocus();
                            chatController.showEmoji.value =
                                !chatController.showEmoji.value;
                          },
                          child: Icon(
                            Icons.emoji_emotions_outlined,
                            color: kDarkNeutralLighten3,
                            size: 18,
                          ),
                        ),
                        SizedBox(
                          width: 15,
                        ),
                        // InkWell(
                        //   onTap: () =>
                        //       chatController.openFileExplorer(),
                        //   child: Icon(
                        //     Icons.attachment_sharp,
                        //     color: kDarkNeutralLighten3,
                        //     size: 18,
                        //   ),
                        // ),
                        // SizedBox(
                        //   width: 15,
                        // ),
                        SizedBox(
                          height: 40.0,
                          width: 40.0,
                          child: FloatingActionButton(
                            onPressed: () {
                              if (controller.text.isEmpty) {
                                return;
                              }
                              FocusScope.of(context).unfocus();
                              chatController.sendNewMessage(
                                  body: controller.text);
                              controller.clear();
                            },
                            backgroundColor: kPrimaryBase,
                            elevation: 0,
                            child: Icon(
                              Icons.send,
                              color: Colors.white,
                              size: 14,
                            ),
                          ),
                        ),
                      ],
                    ),
                  ],
                ),
              ),
              Obx(
                () {
                  if (chatController.showEmoji.isFalse) {
                    return Container();
                  }
                  return SizedBox(
                    height: 250,
                    child: EmojiPicker(
                        onEmojiSelected: (Category category, Emoji emoji) {
                          _onEmojiSelected(emoji);
                        },
                        onBackspacePressed: _onBackspacePressed,
                        config: Config(
                            columns: 7,
                            emojiSizeMax: 32 * (Platform.isIOS ? 1.30 : 1.0),
                            verticalSpacing: 0,
                            horizontalSpacing: 0,
                            initCategory: Category.RECENT,
                            bgColor: const Color(0xFFF2F2F2),
                            indicatorColor: Colors.blue,
                            iconColor: Colors.grey,
                            iconColorSelected: Colors.blue,
                            progressIndicatorColor: Colors.blue,
                            backspaceColor: Colors.blue,
                            showRecentsTab: true,
                            recentsLimit: 28,
                            noRecentsText: 'No Recents',
                            noRecentsStyle: const TextStyle(
                                fontSize: 20, color: Colors.black26),
                            tabIndicatorAnimDuration: kTabScrollDuration,
                            categoryIcons: const CategoryIcons(),
                            buttonMode: ButtonMode.MATERIAL)),
                  );
                },
              ),
            ],
          ),
        )),
      ],
    );
  }
}

class FeelingsWidget extends StatelessWidget {
  final String text;
  final String imagePath;
  final Color? color;
  final bool selected;

  const FeelingsWidget(
      {Key? key,
      this.text = '',
      this.color,
      required this.imagePath,
      this.selected = false})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Opacity(
      opacity: selected ? 1 : 0.5,
      child: Column(
        children: [
          Container(
            height: 40,
            width: 40,
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              color: kPrimaryLighten0,
            ),
            padding: EdgeInsets.all(2),
            child: Image.network(imagePath),
          ),
          SizedBox(
            height: 2,
          ),
          Text(
            text,
            style: TextStyle(
              fontFamily: 'Jost',
              fontSize: 12,
              color: color,
              fontWeight: FontWeight.w400,
            ),
          )
        ],
      ),
    );
  }
}
