import 'package:bento_support_flutter/UI/shared/custom_widgets/support_background.dart';
import 'package:bento_support_flutter/controllers/auth_controller.dart';
import 'package:bento_support_flutter/models/organization.model.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_instance/src/extension_instance.dart';

import 'package:rothko_mobile/rothko.dart';

import 'chat/controllers/chat_controller.dart';

class SupportScreen extends StatefulWidget {
  @override
  _SupportScreenState createState() => _SupportScreenState();
}

class _SupportScreenState extends State<SupportScreen> {
  ScrollController? _scrollController;

  @override
  void initState() {
    _scrollController = ScrollController();
    super.initState();
  }

  final authController = Get.find<AuthController>();

  final chatController = Get.find<ChatController>();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Colors.transparent,
        body: Stack(
          children: <Widget>[
            Positioned.fill(
              top: MediaQuery.of(context).size.height * (6 / 100),
              child: Column(
                children: [
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 22.5),
                    child: RichText(
                      text: TextSpan(
                          text:
                              'We hope you never need us, but if you do, we are always ready to solve your issues.',
                          style: TextStyle(
                              fontWeight: FontWeight.w400,
                              fontSize: 14.0,
                              color: kPrimaryLighten0,
                              fontFamily: 'Jost'),
                          children: [
                            TextSpan(
                                text:
                                    'Select a possible complaint category or send us a custom message.',
                                style: TextStyle(
                                    fontWeight: FontWeight.w600,
                                    fontSize: 14.0,
                                    color: kPrimaryLighten0,
                                    fontFamily: 'Jost'))
                          ]),
                    ),
                  ),
                  Expanded(
                      child: SingleChildScrollView(
                    padding: EdgeInsets.only(top: 35),
                    controller: _scrollController,
                    child: Column(
                      children: <Widget>[
                        _buildOptions(),
                      ],
                    ),
                  ))
                ],
              ),
            )
          ],
        ));
  }

  Widget _optionItem(String title, Function onTap, Widget trailing) {
    var br = BorderRadius.circular(15);
    return Padding(
      padding: EdgeInsets.only(bottom: 20),
      child: Material(
        color: kLightNeutralBase,
        borderRadius: br,
        child: InkWell(
          onTap: () => onTap(),
          borderRadius: br,
          child: Container(
            decoration: BoxDecoration(
              borderRadius: br,
            ),
            child: Padding(
              padding:
                  const EdgeInsets.symmetric(vertical: 18.0, horizontal: 16),
              child: InkWell(
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: <Widget>[
                    Row(
                      children: <Widget>[
                        Text(
                          title,
                          textAlign: TextAlign.left,
                          style: TextStyle(
                              fontFamily: 'Jost',
                              color: Color(0xff042c5c),
                              fontSize: 16,
                              fontWeight: FontWeight.w400),
                        ),
                      ],
                    ),
                    trailing
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildOptions() {
    return Container(
      margin: EdgeInsets.symmetric(horizontal: 24),
      padding: EdgeInsets.only(top: 19, bottom: 19),
      decoration: BoxDecoration(borderRadius: BorderRadius.circular(12)),
      child: SingleChildScrollView(
        child: Obx(() {
          return Column(
              children: ([
            ...authController.organization.value.organizationSetting?.tags ??
                [],
            OrganizationTag(title: 'Custom')
          ])
                  .map((e) => _optionItem(e.title, () {
                        chatController.startConversationView(
                            client: authController.client.value,
                            tags: [e.title]);
                      },
                          Icon(
                            Icons.arrow_forward,
                            size: 30,
                            color: Color(0xff0047CC),
                          )))
                  .toList());
        }),
      ),
    );
  }
}
