import 'dart:async';

import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/controllers/loader.controller.dart';
import 'package:bento_support_flutter/models/client.model.dart';
import 'package:bento_support_flutter/models/complaint.dart';
import 'package:bento_support_flutter/models/organization.model.dart';
import 'package:bento_support_flutter/models/socket_message.dart';
import 'package:bento_support_flutter/models/user.model.dart';
import 'package:bento_support_flutter/services/api/api_services.dart';
import 'package:bento_support_flutter/services/auth.service.dart';
import 'package:bento_support_flutter/services/client.service.dart';
import 'package:get/get.dart';

import 'socket.controller.dart';

class AuthController extends GetxController {
  late Rx<Client<String>> client;

  late RxString publicKey;

  late Rx<Organization> organization;

  late Rx<Responder> orgDefaultResponder;

  RxBool isInitialized = false.obs;

  Future<void> initialize(
      CreateClientDTO? createClientDto, String _publicKey) async {
    publicKey = _publicKey.obs;
    ApiServices.publicKey = _publicKey;
    var chatController = Get.find<ChatController>();

    unawaited(
      LoaderController.instance.loadAsyncProcess(() async {
        var resp = await AuthService.fetchOrganization();
        organization = resp.data.obs;
        orgDefaultResponder =
            organization.value.organizationSetting!.responder.obs;
        chatController.currentAgent = Profile(
          id: '',
          name: orgDefaultResponder.value.name,
          profileImageUrl: orgDefaultResponder.value.avatarUrl,
        ).obs;
        isInitialized.value = true;
        if (createClientDto?.clientReference != null) {
          var clientResp = await ClientService.createClient(createClientDto!);
          client = (clientResp.data as Client<String>).obs;
          chatController.initializeLoginUserView();
        } else {
          chatController.initializeAnonymousUserView(orgDefaultResponder.value);
        }
      }, showError: true),
    );
  }
}
