import 'dart:developer';

import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/controllers/auth_controller.dart';
import 'package:bento_support_flutter/models/socket_message.dart';
import 'package:bento_support_flutter/services/config.dart';
import 'package:bento_support_flutter/services/socket.service.dart';
import 'package:get/get.dart';
import 'package:socket_io_client/socket_io_client.dart' as socketio;
import 'package:socket_io_client/socket_io_client.dart';

class SocketController extends GetxController {
  late socketio.Socket socket;

  List<String> eventList = [ON_MESSAGE];

  void initialize() async {
    var authController = Get.find<AuthController>();
    socket = socketio.io(
        EnvConfig.baseUrl,
        OptionBuilder()
            .setTransports(['websocket', 'polling']) // for Flutter or Dart VM
            .setExtraHeaders(<String, dynamic>{
          'public_key': authController.publicKey.value,
          'client_reference': authController.client.value.clientReference,
        }).build());
    socket.onConnect((dynamic _) {
      setUpEventHandler();
    });
  }

  void setUpEventHandler() {
    eventList.forEach((e) {
      switch (e) {
        case ON_MESSAGE:
          {
            socket.on(e, (dynamic data) {
              Get.find<ChatController>().handleOnMessage(
                  SocketMessage.fromJson(data as Map<String, dynamic>));
            });
            break;
          }
        case ROOM_JOINED:
          {
            break;
          }
        default:
          break;
      }
    });
  }

  void emitEvent(String eventName, SocketMessage message) {
    socket.emit(eventName, message.toMap());
  }
}
