import 'dart:async';

import 'package:bento_support_flutter/UI/shared/custom_widgets/loading_widget.dart';
import 'package:bento_support_flutter/UI/shared/theme/app_theme.dart';
import 'package:bento_support_flutter/UI/view/chat/chat_screen.dart';
import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/controllers/auth_controller.dart';
import 'package:bento_support_flutter/controllers/client_controller.dart';
import 'package:bento_support_flutter/controllers/loader.controller.dart';
import 'package:bento_support_flutter/controllers/socket.controller.dart';
import 'package:bento_support_flutter/controllers/theme_controller.dart';
import 'package:bento_support_flutter/models/user.model.dart';
import 'package:bento_support_flutter/services/config.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:rothko_mobile/rothko.dart';

void main() {
  runApp(MyApp());
  registerControllers();
  RothkoCache.initialize(
      (context) => Get.find<ThemeController>().getCurrentThemeMode(context),
      () => null);
}

var getXInitialized = false;

void registerControllers() {
  getXInitialized = true;
  Get.put(ThemeController());
  Get.put(ClientController());
  Get.put(ChatController());
  Get.put(LoaderController());
  Get.put(AuthController());
  Get.put(SocketController());
}

class MyApp extends StatelessWidget {
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return GetMaterialApp(
      debugShowCheckedModeBanner: false,
      title: 'Support',
      theme: ThemeData(
        primarySwatch: createMaterialColor(kPrimaryBase),
        visualDensity: VisualDensity.adaptivePlatformDensity,
      ),
      home: StartApp(),
    );
  }
}

class StartApp extends StatefulWidget {
  const StartApp({Key? key}) : super(key: key);

  @override
  State<StartApp> createState() => _StartAppState();
}

class _StartAppState extends State<StartApp> {
  @override
  void initState() {
    super.initState();
    Future.delayed(Duration.zero, () {
      initializeBentoSupport(
          onLogin: () {},
          key: 'unauthenticated',
          baseUrl: 'https://supportapi.bento.africa',
          // client: CreateClientDTO(
          //     name: 'Aidrey John',
          //     email: 'audreyjames@testo.com',
          //     clientReference: '6160ioiooioa7d78e589c6c834....oppp55a48',
          //     meta: ClientMeta(companyName: 'Ade')),
          publicKey: 'pk_ocniyemixegolhejxvrhaeyjdvvsaltkttgxziew',
          context: context,
          bottom: 30);
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.white,
    );
  }
}

void removeBentoSupport(String key) {
  var entry = supportWidgetRegister[key];
  if (entry == null) {
    return;
  }
  entry.remove();
  supportWidgetRegister.remove(key);
  getXInitialized = false;
  Get.deleteAll();
}

var supportWidgetRegister = <String, OverlayEntry>{};

void initializeBentoSupport(
    {CreateClientDTO? client,
    required String publicKey,
    required BuildContext context,
    bool expanded = false,
    double? right,
    double? bottom,
    double? left,
    required Function onLogin,
    String? baseUrl,
    Widget? child,
    String? key,
    double? top}) async {
  key ??= 'unknown';

  if (!getXInitialized) {
    registerControllers();
  }

  Get.find<ChatController>().context = context;
  var existingEntry = supportWidgetRegister[key];

  Get.find<ChatController>().expanded.value = expanded;
  if (existingEntry != null) {
    return;
  }
  Get.find<ChatController>().setLoginAction(onLogin);
  await EnvConfig.initialize(baseUrl);

  unawaited(Get.find<AuthController>().initialize(client, publicKey));
  var entry = OverlayEntry(
      builder: (context) => BentoSupportWidget(
            right: right,
            bottom: bottom,
            left: left,
            top: top,
            child: child,
          ));
  supportWidgetRegister.addAll({key: entry});
  Overlay.of(context)?.insert(supportWidgetRegister[key]!);
}

class BentoSupportWidget extends StatelessWidget {
  final double? right;
  final double? bottom;
  final double? left;
  final double? top;
  final Widget? child;

  BentoSupportWidget(
      {this.left, this.right, this.bottom, this.top, this.child}) {}

  final chatController = Get.find<ChatController>();

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      if (chatController.expanded.isTrue) {
        if (LoaderController.instance.appLoading.value) {
          return LoadingWidget();
        }
        return ChatScreen();
      }
      return Positioned(
        right: right ?? (left == null ? 20 : null),
        bottom: bottom ?? (top == null ? 20 : null),
        left: left,
        top: top,
        child: Material(
          child: InkWell(
              borderRadius: BorderRadius.circular(100),
              onTap: () {
                chatController.expanded.value = !chatController.expanded.value;
              },
              child: child ?? Icon(Icons.chat)),
        ),
      );
    });
  }
}
