import 'dart:convert';

import 'package:bento_support_flutter/models/organization.model.dart';

class Client<OrgType> {
  Client({
    required this.isDeleted,
    required this.name,
    required this.email,
    required this.clientReference,
    required this.phoneNumber,
    required this.organization,
    required this.createdAt,
    required this.updatedAt,
    required this.id,
    required this.profileImageUrl,
  });

  bool isDeleted;
  String name;
  String? email;
  String clientReference;
  String? phoneNumber;
  OrgType organization;
  DateTime createdAt;
  DateTime updatedAt;
  String id;
  String profileImageUrl;

  Client copyWith({
    bool? isDeleted,
    String? name,
    String? email,
    String? clientReference,
    String? phoneNumber,
    OrgType? organization,
    DateTime? createdAt,
    DateTime? updatedAt,
    String? id,
    String? profileImageUrl,
  }) =>
      Client<OrgType>(
          isDeleted: isDeleted ?? this.isDeleted,
          name: name ?? this.name,
          email: email ?? this.email,
          clientReference: clientReference ?? this.clientReference,
          phoneNumber: phoneNumber ?? this.phoneNumber,
          organization: organization ?? this.organization,
          createdAt: createdAt ?? this.createdAt,
          updatedAt: updatedAt ?? this.updatedAt,
          id: id ?? this.id,
          profileImageUrl: profileImageUrl ?? this.profileImageUrl);

  factory Client.fromJson(Map<String, dynamic> d) => Client.fromMap(d);

  factory Client.fromMap(Map<String, dynamic> json) => Client(
      isDeleted: json['isDeleted'] as bool,
      name: json['name'] as String,
      email: json['email'] as String?,
      clientReference: json['clientReference'] as String,
      phoneNumber: json['phoneNumber'] as String?,
      organization: (json['organization'].runtimeType == String
          ? (json['organization'])
          : Organization.fromJson(
              json['organization'] as Map<String, dynamic>)) as OrgType,
      createdAt: DateTime.parse(json['createdAt'] as String),
      updatedAt: DateTime.parse(json['updatedAt'] as String),
      id: json['id'] as String,
      profileImageUrl: json['profileImageUrl'] as String);

  Map<String, dynamic> toMap() => <String, dynamic>{
        'isDeleted': isDeleted,
        'name': name,
        'email': email,
        'clientReference': clientReference,
        'phoneNumber': phoneNumber,
        'organization': organization.runtimeType == String
            ? organization
            : (organization as dynamic).toJson(),
        'createdAt': createdAt.toIso8601String(),
        'updatedAt': updatedAt.toIso8601String(),
        'id': id,
        'profileImageUrl': profileImageUrl.toString()
      };
}
