// To parse this JSON data, do
//
//     final organizatioin = organizatioinFromJson(jsonString);

import 'package:meta/meta.dart';
import 'dart:convert';

class Organization {
  Organization({
    required this.isDeleted,
    required this.name,
    required this.email,
    required this.country,
    required this.slug,
    required this.createdAt,
    required this.updatedAt,
    required this.organizationSetting,
    required this.id,
  });

  bool isDeleted;
  String name;
  String email;
  String country;
  String slug;
  DateTime createdAt;
  DateTime updatedAt;
  OrganizationSetting? organizationSetting;
  String id;

  Organization copyWith({
    bool? isDeleted,
    String? name,
    String? email,
    String? country,
    String? slug,
    DateTime? createdAt,
    DateTime? updatedAt,
    OrganizationSetting? organizationSetting,
    String? id,
  }) =>
      Organization(
        isDeleted: isDeleted ?? this.isDeleted,
        name: name ?? this.name,
        email: email ?? this.email,
        country: country ?? this.country,
        slug: slug ?? this.slug,
        createdAt: createdAt ?? this.createdAt,
        updatedAt: updatedAt ?? this.updatedAt,
        organizationSetting: organizationSetting ?? this.organizationSetting,
        id: id ?? this.id,
      );

  factory Organization.fromRawJson(String str) =>
      Organization.fromJson(json.decode(str) as Map<String, dynamic>);

  String toRawJson() => json.encode(toJson());

  factory Organization.fromJson(Map<String, dynamic> json) => Organization(
        isDeleted: json['isDeleted'] as bool,
        name: json['name'] as String,
        email: json['email'] as String,
        country: json['country'] as String,
        slug: json['slug'] as String,
        createdAt: DateTime.parse(json['createdAt'] as String),
        updatedAt: DateTime.parse(json['updatedAt'] as String),
        organizationSetting: OrganizationSetting.fromJson(
            json['organizationSetting'] as Map<String, dynamic>),
        id: json['id'] as String,
      );

  Map<String, dynamic> toJson() => <String, dynamic>{
        'isDeleted': isDeleted,
        'name': name,
        'email': email,
        'country': country,
        'slug': slug,
        'createdAt': createdAt.toIso8601String(),
        'updatedAt': updatedAt.toIso8601String(),
        'organizationSetting': organizationSetting?.toJson(),
        'id': id,
      };
}

class OrganizationSetting {
  OrganizationSetting({
    required this.isDeleted,
    required this.tags,
    required this.organization,
    required this.createdAt,
    required this.updatedAt,
    required this.id,
    required this.responder,
  });

  bool isDeleted;
  List<OrganizationTag> tags;
  String organization;
  DateTime createdAt;
  DateTime updatedAt;
  String id;

  Responder responder;

  OrganizationSetting copyWith({
    bool? isDeleted,
    List<OrganizationTag>? tags,
    String? organization,
    DateTime? createdAt,
    DateTime? updatedAt,
    String? id,
    Responder? responder,
  }) =>
      OrganizationSetting(
          isDeleted: isDeleted ?? this.isDeleted,
          tags: tags ?? this.tags,
          organization: organization ?? this.organization,
          createdAt: createdAt ?? this.createdAt,
          updatedAt: updatedAt ?? this.updatedAt,
          id: id ?? this.id,
          responder: responder ?? this.responder);

  factory OrganizationSetting.fromRawJson(String str) =>
      OrganizationSetting.fromJson(json.decode(str) as Map<String, dynamic>);

  String toRawJson() => json.encode(toJson());

  factory OrganizationSetting.fromJson(Map<String, dynamic> json) =>
      OrganizationSetting(
          isDeleted: json['isDeleted'] as bool,
          tags: List<OrganizationTag>.from((json['tags'] as List).map<dynamic>(
              (dynamic x) =>
                  OrganizationTag.fromMap(x as Map<String, dynamic>))),
          organization: json['organization'] as String,
          createdAt: DateTime.parse(json['createdAt'] as String),
          updatedAt: DateTime.parse(json['updatedAt'] as String),
          id: json['id'] as String,
          responder:
              Responder.fromJson(json['responder'] as Map<String, dynamic>));

  Map<String, dynamic> toJson() => <String, dynamic>{
        'isDeleted': isDeleted,
        'tags': List<dynamic>.from(tags.map<dynamic>((dynamic x) => x)),
        'organization': organization,
        'createdAt': createdAt.toIso8601String(),
        'updatedAt': updatedAt.toIso8601String(),
        'id': id,
        'responder': responder.toJson()
      };
}

class Responder {
  Responder({
    required this.name,
    required this.avatarUrl,
    required this.welcomeMessage,
  });
  late final String name;
  late final String avatarUrl;
  late final String welcomeMessage;

  Responder.fromJson(Map<String, dynamic> json) {
    name = json['name'] as String;
    avatarUrl = json['avatarUrl'] as String;
    welcomeMessage = json['welcomeMessage'] as String;
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['name'] = name;
    _data['avatarUrl'] = avatarUrl;
    _data['welcomeMessage'] = welcomeMessage;
    return _data;
  }
}

class OrganizationTag {
  OrganizationTag({
    required this.title,
    this.defaultMessage,
  });

  String title;
  String? defaultMessage;

  OrganizationTag copyWith({
    String? title,
    String? defaultMessage,
  }) =>
      OrganizationTag(
        title: title ?? this.title,
        defaultMessage: defaultMessage ?? this.defaultMessage,
      );

  factory OrganizationTag.fromMap(Map<String, dynamic> json) => OrganizationTag(
        title: json['title'] as String,
        defaultMessage: json['defaultMessage'] as String?,
      );

  Map<String, dynamic> toMap() => <String, dynamic>{
        'title': title,
        'defaultMessage': defaultMessage,
      };
}
