import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:bento_support_flutter/UI/shared/custom_widgets/loading_widget.dart';
import 'package:bento_support_flutter/UI/view/chat/controllers/chat_controller.dart';
import 'package:bento_support_flutter/controllers/loader.controller.dart';
import 'package:bento_support_flutter/services/parser.service.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:mime/mime.dart';
import 'package:rothko_mobile/rothko.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:timeago/timeago.dart' as timeago;

class SocketMessage {
  SocketMessage({
    required this.room,
    required this.event,
    required this.message,
  });

  String room;
  String event;
  Message message;

  SocketMessage copyWith(
          {String? room,
          String? event,
          String? socketId,
          String? socketType,
          String? socketTypeId,
          Message? message}) =>
      SocketMessage(
          room: room ?? this.room,
          event: event ?? this.event,
          message: message ?? this.message);

  factory SocketMessage.fromJson(Map<String, dynamic> d) =>
      SocketMessage.fromMap(d);

  factory SocketMessage.fromMap(Map<String, dynamic> json) => SocketMessage(
        room: json['room'] as String,
        event: json['event'] as String,
        message:
            Message<dynamic>.fromMap(json['message'] as Map<String, dynamic>),
      );

  Map<String, dynamic> toMap() => <String, dynamic>{
        'room': room,
        'event': event,
        'message': message.toMap(),
      };
}

class Message<SentBy> {
  Message({
    required this.deliveredTo,
    required this.readBy,
    required this.attachments,
    required this.complaint,
    required this.type,
    required this.sender,
    required this.sentBy,
    required this.createdAt,
    required this.updatedAt,
    this.id,
    this.uuid,
    this.body,
    this.review,
  });

  List<dynamic> deliveredTo;
  List<dynamic> readBy;
  List<MessageAttachment> attachments;
  String complaint;
  String type;
  String sender;
  SentBy sentBy;
  DateTime createdAt;
  DateTime updatedAt;
  String? id;
  String? uuid;
  String? body;
  int? review;

  Message<T> copyWith<T>({
    List<dynamic>? deliveredTo,
    List<dynamic>? readBy,
    List<MessageAttachment>? attachments,
    String? complaint,
    String? type,
    String? sender,
    required T sentBy,
    DateTime? createdAt,
    DateTime? updatedAt,
    String? id,
    String? uuid,
    String? body,
    int? review,
  }) =>
      Message<T>(
          deliveredTo: deliveredTo ?? this.deliveredTo,
          readBy: readBy ?? this.readBy,
          attachments: attachments ?? this.attachments,
          complaint: complaint ?? this.complaint,
          type: type ?? this.type,
          sender: sender ?? this.sender,
          sentBy: sentBy,
          createdAt: createdAt ?? this.createdAt,
          updatedAt: updatedAt ?? this.updatedAt,
          id: id ?? this.id,
          uuid: uuid ?? this.uuid,
          review: review ?? this.review,
          body: body ?? this.body);
  factory Message.fromMap(Map<String, dynamic> json) => Message(
      deliveredTo:
          (json['deliveredTo'] as List).map<dynamic>((dynamic x) => x).toList(),
      readBy: (json['readBy'] as List).map<dynamic>((dynamic x) => x).toList(),
      attachments: (json['attachments'] as List<dynamic>)
          .map((dynamic e) =>
              MessageAttachment.fromMap(e as Map<String, dynamic>))
          .toList(),
      complaint: json['complaint'] as String,
      type: json['type'] as String,
      sender: json['sender'] as String,
      sentBy: (json['sentBy'].runtimeType == String
          ? json['sentBy']
          : Profile.fromJson(json['sentBy'] as Map<String, dynamic>)) as SentBy,
      createdAt: DateTime.parse(json['createdAt'] as String),
      updatedAt: DateTime.parse(json['updatedAt'] as String),
      id: json['id'] as String,
      uuid: json['uuid'] as String?,
      body: json['body'] as String?,
      review: json['review'] as int?);

  Map<String, dynamic> toMap() => <String, dynamic>{
        'deliveredTo':
            List<dynamic>.from(deliveredTo.map<dynamic>((dynamic x) => x)),
        'readBy': List<dynamic>.from(readBy.map<dynamic>((dynamic x) => x)),
        'attachments': attachments.map((e) => e.toMap).toList(),
        'complaint': complaint,
        'type': type,
        'sender': sender,
        'sentBy': (sentBy.runtimeType == String
            ? sentBy
            : (sentBy as dynamic).toMap()) as SentBy,
        'createdAt': createdAt.toIso8601String(),
        'updatedAt': updatedAt.toIso8601String(),
        'id': id,
        'uuid': uuid,
        'body': body,
        'review': review
      };
  Widget get senderAvatar => CircleAvatar(
        backgroundImage: sentBy.runtimeType == Profile
            ? NetworkImage((sentBy as Profile).profileImageUrl)
            : null,
      );

  Widget get timeAgoWidget => Text(
        timeago.format(createdAt),
        style: TextStyle(
            fontFamily: 'Jost', fontSize: 10, color: kDarkNeutralLighten5),
      );
}

class Profile {
  Profile({
    required this.profileImageUrl,
    required this.name,
    required this.id,
  });

  String profileImageUrl;
  String name;
  String id;

  Profile copyWith({
    String? profileImageUrl,
    String? name,
    String? id,
  }) =>
      Profile(
        profileImageUrl: profileImageUrl ?? this.profileImageUrl,
        name: name ?? this.name,
        id: id ?? this.id,
      );

  factory Profile.fromJson(Map<String, dynamic> d) => Profile.fromMap(d);

  String toJson() => json.encode(toMap());

  factory Profile.fromMap(Map<String, dynamic> json) => Profile(
        profileImageUrl: (json['profileImageUrl'] ??
                'https://bento-resources.s3.amazonaws.com/illustrations/avatar-kofo.png')
            as String,
        name: json['name'] as String,
        id: json['id'] as String,
      );

  Map<String, dynamic> toMap() => <String, dynamic>{
        'profileImageUrl': profileImageUrl,
        'name': name,
        'id': id,
      };
}

class MessageSender {
  static const String user = 'user';
  static const String client = 'client';
}

class MessageAttachment {
  String filename;

  String? url;

  PlatformFile? file;

  LoadingState loadingState = LoadingState.loading;

  MessageAttachment({required this.filename, this.url, this.file}) {
    if (url != null) {
      loadingState = LoadingState.success;
    }
  }

  static MessageAttachment fromMap(Map<String, dynamic> data) {
    return MessageAttachment(
      filename: data['filename'] as String,
      url: data['url'] as String?,
    );
  }

  Map<String, dynamic> get toMap => <String, dynamic>{
        'filename': filename,
        'url': url,
      };

  bool get isUploaded => url != null;

  bool get fileIsImage => ParserService.fileIsImage(url ?? file?.path ?? '');

  Widget getPreviewWidget(String senderType, BuildContext context) {
    var decoration = BoxDecoration(
      color: kPrimaryDarken3.withOpacity(0.2),
      borderRadius: BorderRadius.circular(8),
    );
    Widget? child;
    if (fileIsImage) {
      child = isUploaded
          ? Image.network(url!)
          : FutureBuilder<Uint8List>(
              future: ParserService.getBytesFromFile(file!),
              builder: (context, snapshot) {
                return Image.memory(snapshot.data!);
              });
    } else {
      child = Center(
        child: Icon(
          Icons.file_present_rounded,
          color: kLightNeutralLighten5,
          size: 50,
        ),
      );
    }

    print('Get.find<ChatController>().context' +
        Get.find<ChatController>().context.toString());

    return Column(
      crossAxisAlignment: senderType == MessageSender.user
          ? CrossAxisAlignment.start
          : CrossAxisAlignment.end,
      children: [
        Stack(
          children: [
            Container(
              width: MediaQuery.of(context).size.width * 0.60,
              height: 150,
              decoration: decoration,
              child: child,
              margin: EdgeInsets.symmetric(vertical: 8),
            ),
            Container(
              width: MediaQuery.of(context).size.width * 0.60,
              height: 150,
              margin: EdgeInsets.symmetric(vertical: 8),
              decoration:
                  decoration.copyWith(color: Colors.black.withOpacity(0.4)),
            ),
            if ([
              LoadingState.loading,
              LoadingState.failed,
              LoadingState.success
            ].contains(loadingState))
              Positioned.fill(
                child: Align(
                    alignment: loadingState == LoadingState.failed
                        ? Alignment.topRight
                        : Alignment.center,
                    child: Builder(builder: (context) {
                      if (loadingState == LoadingState.loading) {
                        return LoadingWidget();
                      } else if (loadingState == LoadingState.failed) {
                        return Icon(Icons.warning_rounded, color: kWarningBase);
                      } else {
                        return IconButton(
                            onPressed: () {
                              launch(url!);
                            },
                            icon: Icon(
                              Icons.download_rounded,
                              color: kLightNeutralLighten5,
                            ));
                      }
                    })),
              ),
          ],
        ),
        Text(
          filename,
          overflow: TextOverflow.ellipsis,
          style: TextStyle(fontSize: 10, fontFamily: 'jost'),
        )
      ],
    );
  }

  String get mimeType => lookupMimeType(filename) ?? 'application/octet-stream';
}
