import 'package:bento_support_flutter/models/pagination.dart';

/// A data class used to provide a general structure to api reponses.
class ApiResponse<T> {
  const ApiResponse(
      {required this.success, this.message, this.meta, required this.data});

  /// Whether or not the api call was successful.
  final bool success;

  /// The meta header sent with the response payload.
  final dynamic meta;

  /// An optional message attached to the response payload from
  /// the api call.
  final String? message;

  /// The data itself being requested.
  final T data;

  /// construct an [ApiResponse] instance from a [Map] input.
  static ApiResponse fromJson<TI>(Map<String, dynamic> json) {
    return ApiResponse<TI>(
      success: false,
      meta: json['meta'] == null
          ? null
          : PaginationMetadata.fromJson(json['meta'] as Map<String, dynamic>),
      message: json['message'] as String,
      data: json['data'] as TI,
    );
  }

  /// An error [ApiResponse].
  static const ApiResponse<dynamic> error =
      ApiResponse<dynamic>(success: false, data: null);

  /// Create a copy of this user that with the provided property overriden
  /// if non-null.
  ApiResponse<TO> copyWith<TO>(
      {bool? success, String? message, required TO data}) {
    var _data = data;

    if (_data == null && T is TO) {
      _data = this.data as TO;
    }

    return ApiResponse<TO>(
      success: success ?? this.success,
      meta: meta,
      message: message ?? this.message,
      data: _data,
    );
  }
}
