import 'dart:convert';
import 'dart:developer';

import 'package:bento_support_flutter/models/complaint.dart';
import 'package:bento_support_flutter/models/dtos/create_complaint_dto.dart';
import 'package:bento_support_flutter/models/organization.model.dart';
import 'package:bento_support_flutter/models/pagination.dart';
import 'package:bento_support_flutter/models/socket_message.dart';
import 'package:bento_support_flutter/services/api/api_response.dart';
import 'package:bento_support_flutter/services/api/api_services.dart';

class MessageService {
  static Future<ApiResponse<List<Message<Profile>>>> fetchMessages(
      String clientId, PaginationMetadata meta) {
    var queryMap = <String, dynamic>{
      'page': meta.page + 1,
      'limit': meta.perPage,
    };
    var queryString =
        queryMap.entries.map((e) => '${e.key}=${e.value}').join('&');
    return ApiServices.get(
        path: '/message/client/$clientId?$queryString',
        toModel: (dynamic d) {
          return (d as List)
              .map((dynamic e) =>
                  Message<Profile>.fromMap(e as Map<String, dynamic>))
              .toList();
        });
  }

  static Future<ApiResponse<String>> uploadFile(
      String base64, String fileName, String extension, String mime) {
    return ApiServices.post(
        path: '/file/upload',
        data: <String, dynamic>{
          'data': base64,
          'mime': mime,
          'extension': extension,
          'filename': fileName
        },
        toModel: (dynamic e) => e as String);
  }
}
